@@ -45,6 +45,7 @@ struct _GsScreenshotImagePrivate
 	gchar		*cachedir;
 	gchar		*filename;
 	const gchar	*current_image;
+	gboolean	 use_desktop_background;
 	guint		 width;
 	guint		 height;
 	gint		 scale;
@@ -113,14 +114,18 @@ gs_screenshot_image_get_desktop_pixbuf (GsScreenshotImage *ssimg)
  * gs_screenshot_image_use_desktop_background:
  **/
 static gboolean
-gs_screenshot_image_use_desktop_background (GdkPixbuf *pixbuf)
+gs_screenshot_image_use_desktop_background (GsScreenshotImage *ssimg, GdkPixbuf *pixbuf)
 {
+	GsScreenshotImagePrivate *priv = gs_screenshot_image_get_instance_private (ssimg);
 	AsImage *im;
 	gboolean ret;
 
 	/* nothing to show, means no background mode */
 	if (pixbuf == NULL)
 		return FALSE;
+	/* background mode explicitly disabled */
+	if (!priv->use_desktop_background)
+		return FALSE;
 
 	/* use a temp AsImage */
 	im = as_image_new ();
@@ -148,11 +153,11 @@ as_screenshot_show_image (GsScreenshotImage *ssimg)
 	} else {
 		/* this is always going to have alpha */
 		pixbuf = gdk_pixbuf_new_from_file_at_scale (priv->filename,
-							    priv->width,
-							    priv->height,
+							    priv->width * priv->scale,
+							    priv->height * priv->scale,
 							    FALSE, NULL);
 		if (pixbuf != NULL) {
-			if (gs_screenshot_image_use_desktop_background (pixbuf)) {
+			if (gs_screenshot_image_use_desktop_background (ssimg, pixbuf)) {
 				pixbuf_bg = gs_screenshot_image_get_desktop_pixbuf (ssimg);
 				if (pixbuf_bg == NULL) {
 					pixbuf_bg = g_object_ref (pixbuf);
@@ -201,15 +206,16 @@ gs_screenshot_image_complete_cb (SoupSession *session,
 				 SoupMessage *msg,
 				 gpointer user_data)
 {
-	AsImage *im = NULL;
-	GsScreenshotImagePrivate *priv;
 	GsScreenshotImage *ssimg = GS_SCREENSHOT_IMAGE (user_data);
+	GsScreenshotImagePrivate *priv = gs_screenshot_image_get_instance_private (ssimg);
+	AsImage *im = NULL;
 	GError *error = NULL;
 	GdkPixbuf *pixbuf = NULL;
 	GInputStream *stream = NULL;
 	gboolean ret;
 
-	if (msg->status_code == SOUP_STATUS_CANCELLED)
+	/* return immediately if the message was cancelled or if we're in destruction */
+	if (msg->status_code == SOUP_STATUS_CANCELLED || priv->session == NULL)
 		goto out;
 
 	if (msg->status_code != SOUP_STATUS_OK) {
@@ -220,8 +226,6 @@ gs_screenshot_image_complete_cb (SoupSession *session,
 		goto out;
 	}
 
-	priv = gs_screenshot_image_get_instance_private (ssimg);
-
 	/* create a buffer with the data */
 	stream = g_memory_input_stream_new_from_data (msg->response_body->data,
 						      msg->response_body->length,
@@ -239,8 +243,8 @@ gs_screenshot_image_complete_cb (SoupSession *session,
 
 	/* is image size destination size unknown or exactly the correct size */
 	if (priv->width == G_MAXUINT || priv->height == G_MAXUINT ||
-	    (priv->width == (guint) gdk_pixbuf_get_width (pixbuf) &&
-	     priv->height == (guint) gdk_pixbuf_get_height (pixbuf))) {
+	    (priv->width * priv->scale == (guint) gdk_pixbuf_get_width (pixbuf) &&
+	     priv->height * priv->scale == (guint) gdk_pixbuf_get_height (pixbuf))) {
 		ret = g_file_set_contents (priv->filename,
 					   msg->response_body->data,
 					   msg->response_body->length,
@@ -256,7 +260,8 @@ gs_screenshot_image_complete_cb (SoupSession *session,
 		im = as_image_new ();
 		as_image_set_pixbuf (im, pixbuf);
 		ret = as_image_save_filename (im, priv->filename,
-					      priv->width, priv->height,
+					      priv->width * priv->scale,
+					      priv->height * priv->scale,
 					      AS_IMAGE_SAVE_FLAG_PAD_16_9, &error);
 		if (!ret) {
 			gs_screenshot_image_set_error (ssimg, error->message);
@@ -329,6 +334,18 @@ gs_screenshot_image_set_size (GsScreenshotImage *ssimg,
 	gtk_widget_set_size_request (priv->stack, width, height);
 }
 
+/**
+ * gs_screenshot_image_set_use_desktop_background:
+ **/
+void
+gs_screenshot_image_set_use_desktop_background (GsScreenshotImage *ssimg,
+                                                gboolean use_desktop_background)
+{
+	GsScreenshotImagePrivate *priv = gs_screenshot_image_get_instance_private (ssimg);
+	g_return_if_fail (GS_IS_SCREENSHOT_IMAGE (ssimg));
+	priv->use_desktop_background = use_desktop_background;
+}
+
 /**
  * gs_screenshot_image_load_async:
  **/
@@ -353,18 +370,15 @@ gs_screenshot_image_load_async (GsScreenshotImage *ssimg,
 	g_return_if_fail (priv->width != 0);
 	g_return_if_fail (priv->height != 0);
 
-	/* load the HiDPI image if it exists */
-	priv->scale = 1;
-	if (gtk_widget_get_scale_factor (GTK_WIDGET (ssimg)) > 1) {
-		im = as_screenshot_get_image (priv->screenshot,
-					      priv->width * 2,
-					      priv->height * 2);
-		if (im != NULL)
-			priv->scale *= 2;
-	}
+	/* load an image according to the scale factor */
+	priv->scale = gtk_widget_get_scale_factor (GTK_WIDGET (ssimg));
+	im = as_screenshot_get_image (priv->screenshot,
+				      priv->width * priv->scale,
+				      priv->height * priv->scale);
 
-	/* fallback to LoDPI if it exists */
-	if (im == NULL) {
+	/* if we've failed to load a HiDPI image, fallback to LoDPI */
+	if (im == NULL && priv->scale > 1) {
+		priv->scale = 1;
 		im = as_screenshot_get_image (priv->screenshot,
 					      priv->width,
 					      priv->height);
@@ -380,7 +394,7 @@ gs_screenshot_image_load_async (GsScreenshotImage *ssimg,
 	if (priv->width == G_MAXUINT || priv->height == G_MAXUINT) {
 		sizedir = g_strdup ("unknown");
 	} else {
-		sizedir = g_strdup_printf ("%ux%u", priv->width, priv->height);
+		sizedir = g_strdup_printf ("%ux%u", priv->width * priv->scale, priv->height * priv->scale);
 	}
 	cachedir = g_build_filename (priv->cachedir,
 				     "gnome-software",
@@ -476,6 +490,7 @@ gs_screenshot_image_init (GsScreenshotImage *ssimg)
 	AtkObject *accessible;
 
 	ssimg->priv = gs_screenshot_image_get_instance_private (ssimg);
+	ssimg->priv->use_desktop_background = TRUE;
 
 	gtk_widget_set_has_window (GTK_WIDGET (ssimg), FALSE);
 	gtk_widget_init_template (GTK_WIDGET (ssimg));
